#|
 Arxiu: cl-gnuplot.lisp
 Contingut: mdul per a controlar gnuplot desde lisp
 Autor: Felip Alez Nadal
 ltima actualitaci: 8-VI-2006 |#


#|

    cl-gnuplot
    Copyright (C) Felip Alez Nadal 2006

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  USA

|#

(declaim (optimize (speed 3) (safety 0) (debug 0)))

(provide :cgn)

(defpackage :cgn
  (:use :cl :ltk)
  (:export #:start-gnuplot #:format-gnuplot #:close-gnuplot #:set-title #:set-grid #:on #:off #:plot-function #:replot-function #:x #:y #:set-range #:plot-points #:postscript-copy #:w32) )

(in-package :cgn)


(defvar *gnuplot* nil "Gnuplot's connection")

(defun start-gnuplot ()
  "Runs gnuplot as a subprocess. This should be the first function to call."
  (cond
    ((eq *gnuplot* nil) (setq *gnuplot* (do-execute "gnuplot" nil)))
    ( t nil))
  (values)
  )


(defun format-gnuplot (text)
  "Sends a command to gnuplot"
  (cond
    ((eq *gnuplot* nil)
     (format t "Gnuplot is not running"))
    (t
     (format *gnuplot* "~A ~%" text)
     (force-output *gnuplot*)))
  
     (values)
     )

(defun close-gnuplot ()
  "Closes gnuplot"
  (cond 
    ( (eq *gnuplot* nil) t)
    ( t 
     (format-gnuplot "quit" )
     (close *gnuplot*) 
     (setq *gnuplot* nil)))
  (values)
  )


(defun update ()
  "Refreshes the screen. This should not be called by the user."
  (format-gnuplot "replot"))

(defun set-title (text)
  "Sets the window title"
  (cond 
    ((eq *gnuplot* nil) )
    ( t
     (format-gnuplot (format nil "set title '~A'" text))
     (update)))
  (values))

(defun set-grid (on-off)
  "Activates/deactivates the grid."
  (cond 
    ( (eq *gnuplot* nil))
    ( t
     (cond
       ((eq on-off 'on) 
	(format-gnuplot "set grid")
	(update)
	)
       ((eq on-off 'off) 
	(format-gnuplot "unset grid")
	(update)
	)
       (t (format t "I don't understand~%")))
     ))
  (values))


(defun plot-function (funcio)
  "Shows a 1-variable function."
  (format-gnuplot (format nil "plot ~A" funcio))
  (values))

(defun replot-function (funcio)
  "Shows a n-variable function on gnuplot / replot version. '"
  (format-gnuplot (format nil "replot ~A" funcio))
  (values))

(defun set-range ( eix min max)
  "Sets the x or y range "
  (cond
    ((eq eix 'x)
     (format-gnuplot (format nil "set xrange [~D:~D]" min max))
     (update))
    ((eq eix 'y)
     (format-gnuplot (format nil "set yrange [~D:~D]" min max))
     (update))
    (t (format t "I don't understand~%")))
  (values))


(defun plot-points ( x y &key x_error y_error)
  "This functions shows scatter plots, with x and y errors if desired"
  (cond 
    ((eq x nil) ; Error
     (format t "I can't plot a serie without x values~%")
     (return-from plot-points (values)))
    ((eq y nil) ; Error
     (format t "I can't plot a serie without y values~%")
     (return-from plot-points (values)))
    ( t ;Si els casos anteriors no han sortit, tenim una srie x i una y -> podem fer el grfic. Ara cal que ens preguntem pels errors
     (format t "Dins~%")
     (cond
       ((and x_error y_error) ;Tenim barres d'error en les x i en les y
	(let (
	      (l_x (length x))
	      (l_y (length y))
	      (l_ey (length y_error))
	      (l_ex (length x_error))
	      len)
	  (setq len (min l_x l_y l_ey l_ex))
	  (with-open-file (stream  ".cl-gnuplot.gn" :direction :output  :if-does-not-exist :create :if-exists :overwrite )
	    (do ((i 0 (incf i)))
		((= i len) t)
	      (format stream "~D        ~D      ~D      ~D  ~%" (nth i x ) (nth i y) (nth i x_error) (nth i y_error))
	      ))
	  (format-gnuplot "plot '.cl-gnuplot.gn' with xyerrorbars")
	  ))
       ( x_error ;Tenim barres d'error en les x noms
	(let (
	      (l_x (length x))
	      (l_y (length y))
	      (l_ex (length x_error))
	      len)
	  (setq len (min l_x l_y l_ex))
	  (with-open-file (stream  ".cl-gnuplot.gn" :direction :output  :if-does-not-exist :create :if-exists :overwrite )
	    (do ((i 0 (incf i)))
		((= i len ) t)
	      (format stream "~D        ~D      ~D~%" (nth i x ) (nth i y) (nth i x_error))
	      ))
	  (format-gnuplot "plot '.cl-gnuplot.gn' with xerrorbars")
	  ))
       
       ( y_error ;Tenim barres d'error en les y noms
	(let (
	      (l_x (length x))
	      (l_y (length y))
	      (l_ey (length y_error))
	      len)
	  (setq len (min l_x l_y l_ey))
	  (with-open-file (stream  ".cl-gnuplot.gn" :direction :output  :if-does-not-exist :create :if-exists :overwrite )
	    (do ((i 0 (incf i)))
		((= i len ) t)
	      (format stream "~D        ~D      ~D~%" (nth i x ) (nth i y) (nth i y_error))
	      ))
	  (format-gnuplot "plot '.cl-gnuplot.gn' with yerrorbars")
	))
       ( t ; No tenim barres d'error
	(let (
	       (l_x (length x))
	      (l_y (length y))
	       len)
	  (setq len (min l_x l_y))
	  (with-open-file (stream  ".cl-gnuplot.gn" :direction :output  :if-does-not-exist :create :if-exists :overwrite )
	    (do ((i 0 (incf i)))
		((= i len ) t)
	      (format stream "~D        ~D  ~%" (nth i x ) (nth i y))
	      ))
	  (format-gnuplot "plot '.cl-gnuplot.gn'")
	  ))
	))
    )
  )


(defun postscript-copy ( filename os )
  "Saves a postscript copy of the screen . You must specify you os, mainly 'w32 if you use windows. For another os, you must use 'unix."
  (format-gnuplot "set terminal postscript color")
  (format-gnuplot (format nil "set output '~D'" filename))
  (update)
  ;Torne a posar la terminal normal. Ull!! A pot provocar problemes en windows, provar-ho!
  (cond
    ((eq os 'w32)
     (format-gnuplot "set terminal windows"))
    ( t 
     (format-gnuplot "set terminal x11")))
  (values))
